unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, VirtualTrees;

type

  TKapitelArt = (kaKapitel, kaUnterKapitel);

  TKapitel = record
    Titel: String;
    Art: TKapitelArt;
    Gedruckt: Boolean;
    Seitenzahl: Double;
    BeginntAufSeite: Integer;
    BegonnenAm: TDateTime;
    Fett: Boolean;
    Perspektive: Integer;
    InfoIndex: Integer;
  end;
  PKapitel = ^TKapitel;

  { TForm1 }

  TForm1 = class(TForm)
    ImageList1: TImageList;
    ImageList2: TImageList;
    KapitelTree: TVirtualStringTree;
    procedure FormCreate(Sender: TObject);
    procedure KapitelTreeAfterCellPaint(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      const CellRect: TRect);
    procedure KapitelTreeDrawText(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      const CellText: String; const CellRect: TRect; var DefaultDraw: Boolean);
    procedure KapitelTreeFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure KapitelTreeGetImageIndex(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
      var Ghosted: Boolean; var ImageIndex: Integer);
    procedure KapitelTreeGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
  private
    function AddKapitel(AParentNode: PVirtualNode; ATitel: String;
      Art: TKapitelArt; Perspektive: Integer; Gedruckt, Fett: Boolean;
      Seitenzahl: Double; BeginntAufSeite: Integer;
      BegonnenAm: TDateTime; InfoIndex: Integer): PVirtualNode;
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses
  LCLIntf;

{ TForm1 }

{ Helfer-Routine, im die Baum-Knoten zu füllen. Bekommt alle benötigten Parameter
  als Argumente. Diese werden in den TKapitel-Record geschrieben, der vom Tree
  verwaltet wird (nicht sauber "virtual paradigm"...), aber sehr einfach. }
function TForm1.AddKapitel(AParentNode: PVirtualNode; ATitel: String;
  Art: TKapitelArt; Perspektive: Integer; Gedruckt, Fett: Boolean;
  Seitenzahl: Double; BeginntAufSeite:
  Integer; BegonnenAm: TDateTime; InfoIndex: Integer): PVirtualNode;
var
  kap: PKapitel;
begin
  Result := Kapiteltree.AddChild(AParentNode);
  kap := Kapiteltree.GetNodeData(Result);
  kap^.Titel := ATitel;
  kap^.Art := Art;
  kap^.Perspektive := Perspektive;
  kap^.Gedruckt := Gedruckt;
  kap^.Fett := Fett;
  kap^.Seitenzahl := Seitenzahl;
  kap^.BeginntAufSeite := BeginntAufSeite;
  kap^.BegonnenAm := BegonnenAm;
  kap^.InfoIndex := InfoIndex;
  if AParentNode <> nil then
    KapitelTree.Expanded[AParentNode] := true;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  N: PVirtualNode;
  P: PKapitel;
begin
  // Dies kann man auch alles im Objekt-Inspektor erledigen. Schreibe es hier,
  // damit man sieht, was geändert wurde.
  KapitelTree.NodeDataSize := SizeOf(TKapitel);
  KapitelTree.TreeOptions.MiscOptions := KapitelTree.TreeOptions.MiscOptions + [toGridExtensions];
  KapitelTree.TreeOptions.PaintOptions := KapitelTree.TreeOptions.PaintOptions - [toThemeAware];
  KapitelTree.TreeOptions.SelectionOptions := KapitelTree.TreeOptions.SelectionOptions + [toFullRowSelect];
  KapitelTree.Header.Height := 24;
  KapitelTree.Header.Images := ImageList1;
  KapitelTree.Header.Columns.Clear;
  with KapitelTree.Header.Columns.Add do begin      // 0: Kapitel
    Text := 'Kapitel / Szene';
    Width := 300;
  end;
  with KapitelTree.Header.Columns.Add do begin      // 1: Art
    Text := '';  // Kapitel
    ImageIndex := 0;
    Width := 26;
  end;
  with KapitelTree.Header.Columns.Add do begin      // 2: Info
    Text := '';  // Info
    CaptionAlignment := taCenter;
    ImageIndex := 7;
    Width := 64;
  end;
  with KapitelTree.Header.columns.Add do begin      // 3: Perspektive
    Text := 'Perspektive';
    Width := 80;
  end;
  with KapitelTree.Header.Columns.Add do begin      // 4: Gedruckt
    Text := '';  // Gedruckt
    Width := 26;
    ImageIndex := 2;
  end;
  with KapitelTree.Header.Columns.Add do begin      // 5: Seitenzahl
    Text := '';  // Seitenzahl
    Alignment := taCenter;
    CaptionAlignment := taCenter;
    Width := 40;
    ImageIndex := 3;
  end;
  with KapitelTree.Header.Columns.Add do begin      // 6: Begonnen am
    Text := 'von...';
    Width := 80;
    ImageIndex := 4;
  end;
  with KapitelTree.Header.Columns.Add do begin      // 7: Seite
    Text := '';
    Width := 40;
    Alignment := taCenter;
    CaptionAlignment := taCenter;
    ImageIndex := 3;
  end;
  KapitelTree.Header.Options := KapitelTree.Header.Options + [hoVisible, hoShowImages];

  // Ab hier wird der Baum aufgebaut
  N := AddKapitel(nil, '1. Kapitel', kaKapitel, 80, false, true, 0.6, 1, EncodeDate(2021,5,2), 0);
  N := AddKapitel(nil, '2. Kapitel & viel Buntes', kaKapitel, 30, false, true, 1.1, 2, Encodedate(2021,5,7), 1);
  N := AddKapitel(nil, '3. Kapitel (No 3)', kaKapitel, 85, false, true, 0.8, 3, EncodeDate(2021, 5, 9), 2);
  N := AddKapitel(nil, '4. Kapitel', kaUnterKapitel, 50, false, false, 0.2, 3, EncodeDate(2021,5,12), 3);
  N := AddKapitel(N, 'Unterkapitel mit etwas Text', kaUnterkapitel, 50, false, false, 0.0, 3, EncodeDate(2021, 5, 14), -1);
  N := AddKapitel(nil, 'Glossar', kaKapitel, 30, false, true, 0.0, 3, EncodeDate(2021,5, 17), 4);
end;

// Dieses Event wird aufgerufen, wenn eine Zelle fertig gezeichnet ist. Das
// Event wird hier "missbraucht", um die langen "Info"-Bilder drüber zu malen.
procedure TForm1.KapitelTreeAfterCellPaint(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  const CellRect: TRect);
var
  kap: PKapitel;
begin
  if Column = 2 then begin
    kap := Sender.GetNodeData(Node);
    if (kap <> nil) and (kap^.InfoIndex > -1) then
      ImageList2.Draw(TargetCanvas, CellRect.Left, CellRect.Top, kap^.InfoIndex);
  end;
end;

{ Wird aufgerufen, bevor der Text in einer Zelle geschrieben wird.
  Wird hier überschrieben, um die "fetten" Zellen festzulegen, und auch
  um den "Balken-Graph" in deri Perspektive-Spalte zu zeichen. }
procedure TForm1.KapitelTreeDrawText(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  const CellText: String; const CellRect: TRect; var DefaultDraw: Boolean);
var
  kap: PKapitel;
  R: TRect;
begin
  kap := Sender.GetNodeData(Node);
  if kap^.Fett and not (Column in [3, 6]) then
    TargetCanvas.Font.Style := [fsBold]
  else
    TargetCanvas.Font.Style := [];

  if (Column = 3) and (kap^.Perspektive > 0) then
  begin
    R := Sender.GetDisplayRect(Node, Column, false);
    R := Rect(R.Left+1, CellRect.Top+1, R.Left + CellRect.Width * kap^.Perspektive div 100, CellRect.Bottom-1);
    TargetCanvas.Brush.Style := bsSolid;
    TargetCanvas.Brush.Color := clSilver;
    TargetCanvas.FillRect(R);
  end;
  DefaultDraw := true;
end;

procedure TForm1.KapitelTreeFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  kap: PKapitel;
begin
  kap := KapitelTree.GetNodeData(Node);
  kap^.Titel := '';  // Den String freigeben, sonst hat man ein Speicherleck.
end;

{ Hier werden die Image-Indices der regulären Bilder im StringTree abgefragt. }
procedure TForm1.KapitelTreeGetImageIndex(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
  var Ghosted: Boolean; var ImageIndex: Integer);
var
  kap: PKapitel;
begin
  kap := KapitelTree.GetNodeData(Node);
  case Column of
    1: case kap^.Art of
         kaKapitel: ImageIndex := 0;
         kaUnterKapitel: ImageIndex := 1;
       end;
    4: if kap^.Gedruckt then ImageIndex := 6 else ImageIndex := 5;
  end;
end;

{ Dieser Ereignis bestimmt aus den Node-Daten, welcher Text für welche Zelle
  auszugeben ist. }
procedure TForm1.KapitelTreeGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: String);
var
  kap: PKapitel;
begin
  CellText := '';
  kap := KapitelTree.GetNodeData(Node);
  if kap = nil then
    exit;
  case Column of
    0: CellText := kap^.Titel;
    1: ;
    2: ;
    3: if kap^.Perspektive > 0 then CellText := 'P';
    4: ;
    5: CellText := FormatFloat('0.0', kap^.Seitenzahl);
    6: CellText := DateToStr(kap^.BegonnenAm);
    7: CellText := IntToStr(kap^.BeginntAufSeite);
  end;
end;

end.

